import org.apache.pdfbox.exceptions.COSVisitorException;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.common.PDRectangle;
import org.apache.pdfbox.pdmodel.edit.PDPageContentStream;
import org.apache.pdfbox.pdmodel.font.PDFont;
import org.apache.pdfbox.pdmodel.font.PDType1Font;
import org.apache.pdfbox.util.PDFMergerUtility;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.IOException;
import java.util.List;

public class PdfManipulator {
    public static byte[] generateHeaderAndFooter(final byte[] pdf, final String header,
                                                 final boolean includePageNumbers) {
        PDDocument doc = null;
        try {
            doc = load(pdf);
            stamp(doc, header, includePageNumbers);
            return toByteArray(doc);
        } finally {
            closeSilently(doc);
        }

    }

    private static void stamp(final PDDocument doc, final String header, final boolean includePageNumbers) {
        final PDFont font = PDType1Font.TIMES_ROMAN;
        final List allPages = doc.getDocumentCatalog().getAllPages();
        for (int i = 0; i < allPages.size(); i++) {
            PDPage page = (PDPage) allPages.get(i);
            final int pageNumber = i + 1;
            PDRectangle rectangle = page.findMediaBox();
            PDPageContentStream contentStream = null;
            try {
                final boolean appendContent = true;
                final boolean compress = true;
                final boolean resetGraphicContext = true;
                contentStream = new PDPageContentStream(doc, page, appendContent, compress, resetGraphicContext);
                setHeader(rectangle, contentStream, font, header, page.getRotation());
                if (includePageNumbers) {
                    setPageNumber(rectangle, contentStream, font, pageNumber, allPages.size(), page.getRotation());
                }
            } catch (IOException e) {
                final String msg = "Feil ved stempling av PDF document.";
                throw new RuntimeException(msg, e);
            } finally {
                closeSilently(contentStream);
            }
        }
    }

    private static void closeSilently(Closeable c) {
        if (c == null) {
            return;
        }
        try {
            c.close();
        } catch (Exception e) {
        }
    }


    private static byte[] toByteArray(PDDocument doc) {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        try {
            try {
                doc.save(output);
            } catch (COSVisitorException | IOException e) {
                throw new RuntimeException("Feil ved skriving av PDF dokument til output stream.", e);
            }
            return output.toByteArray();
        } finally {
            closeSilently(output);
        }
    }

    private static PDDocument load(byte[] pdf) {
        ByteArrayInputStream pdfStream = new ByteArrayInputStream(pdf);
        try {
            try {
                return PDDocument.loadNonSeq(pdfStream, null);
            } catch (IOException e) {
                throw new RuntimeException("Could not load PDF document. Document is probably invalid.", e);
            }
        } finally {
            closeSilently(pdfStream);
        }
    }


    private static void setHeader(PDRectangle rectangle, PDPageContentStream contentStream, PDFont font,
                                  String header, Integer rotation) throws IOException {

        if (rotation == null) {
            rotation = 0;
        }

        contentStream.beginText();
        final int fontSize = 10;
        contentStream.setFont(font, fontSize);

        double rotationInRadian = Math.toRadians(rotation.intValue());

        switch (rotation.intValue()) {
            case 90:
                contentStream.setTextRotation(rotationInRadian, rectangle.getLowerLeftX() + 10,
                        rectangle.getLowerLeftY() + 5);
                break;
            case 180:
                contentStream.setTextRotation(rotationInRadian, rectangle.getUpperRightX() - 5, rectangle
                        .getLowerLeftY() + 10);
                break;
            case 270:
                contentStream.setTextRotation(rotationInRadian, rectangle.getUpperRightX() - 10,
                        rectangle.getUpperRightY() - 5);
                break;
            default:
                contentStream.setTextRotation(0, rectangle.getLowerLeftX() + 5, rectangle.getUpperRightY() - 10);
                break;
        }
        contentStream.drawString(header);
        contentStream.endText();
    }

    private static void setPageNumber(PDRectangle rectangle,
                                      PDPageContentStream contentStream, PDFont font, int page, int totalPages,
                                      Integer rotation) throws IOException {

        if (rotation == null) {
            rotation = 0;
        }

        double rotationInRadian = Math.toRadians(rotation.intValue());

        String text = "Side " + page + " av " + totalPages;
        contentStream.beginText();
        int fontSize = 10;
        contentStream.setFont(font, fontSize);
        float titleWidth = font.getStringWidth(text) / 1000 * fontSize;

        switch (rotation.intValue()) {
            case 90:
                contentStream.setTextRotation(rotationInRadian, rectangle.getLowerLeftX() + 10,
                        rectangle.getUpperRightY() - titleWidth - 5);
                break;
            case 180:
                contentStream.setTextRotation(rotationInRadian, rectangle.getLowerLeftX() + titleWidth + 5, rectangle
                        .getLowerLeftY() + 10);
                break;
            case 270:
                contentStream.setTextRotation(rotationInRadian, rectangle.getUpperRightX() - 10,
                        rectangle.getLowerLeftY() + titleWidth + 5);
                break;
            default:
                contentStream.setTextRotation(0, rectangle.getUpperRightX() - titleWidth - 5, rectangle
                        .getUpperRightY() - 10);
                break;
        }


        contentStream.drawString(text);
        contentStream.endText();
    }


}
